// -*- C++ -*-
/*!
 * @file  Pi_RTIMU.cpp
 * @brief IMU Component using RTIMULib
 * @date $Date$
 *
 * $Id$
 */

#include "Pi_RTIMU.h"
#include "RTIMULib.h"

int sampleCount;
int sampleRate;
uint64_t rateTimer;
uint64_t displayTimer;
uint64_t now;

//  using RTIMULib here allows it to use the .ini file generated by RTIMULibDemo.

RTIMUSettings *settings;
RTIMU *imu;
RTPressure *pressure;
RTHumidity *humidity;

// Module specification
// <rtc-template block="module_spec">
static const char* pi_rtimu_spec[] =
  {
    "implementation_id", "Pi_RTIMU",
    "type_name",         "Pi_RTIMU",
    "description",       "IMU Component using RTIMULib",
    "version",           "1.0.0",
    "vendor",            "MEDIA DRIVE",
    "category",          "Sensor",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.Inifile", "./RTIMULib",

    // Widget
    "conf.__widget__.Inifile", "text",
    // Constraints

    "conf.__type__.Inifile", "string",

    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
Pi_RTIMU::Pi_RTIMU(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_accelOut("accel", m_accel),
    m_compassOut("compass", m_compass),
    m_gyroOut("gyro", m_gyro),
    m_temperatureOut("temperature", m_temperature),
    m_poseOut("pose", m_pose),
    m_pressureOut("pressure", m_pressure),
    m_humidityOut("humidity", m_humidity),
    m_qposeOut("qpose", m_qpose)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
Pi_RTIMU::~Pi_RTIMU()
{
}



RTC::ReturnCode_t Pi_RTIMU::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  addOutPort("accel", m_accelOut);
  addOutPort("compass", m_compassOut);
  addOutPort("gyro", m_gyroOut);
  addOutPort("temperature", m_temperatureOut);
  addOutPort("pose", m_poseOut);
  addOutPort("pressure", m_pressureOut);
  addOutPort("humidity", m_humidityOut);
  addOutPort("qpose", m_qposeOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("Inifile", m_Inifile, "./RTIMULib");
  // </rtc-template>
  
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t Pi_RTIMU::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Pi_RTIMU::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Pi_RTIMU::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t Pi_RTIMU::onActivated(RTC::UniqueId ec_id)
{
    sampleCount = 0;
    sampleRate = 0;

    //  using RTIMULib here allows it to use the .ini file generated by RTIMULibDemo.

    settings = new RTIMUSettings("RTIMULib");

    imu = RTIMU::createIMU(settings);
    pressure = RTPressure::createPressure(settings);
    humidity = RTHumidity::createHumidity(settings);

    if ((imu == NULL) || (imu->IMUType() == RTIMU_TYPE_NULL)) {
        printf("No IMU found\n");
        return RTC::RTC_ERROR;
    }

    //  This is an opportunity to manually override any settings before the call IMUInit

    //  set up IMU

    imu->IMUInit();

    //  this is a convenient place to change fusion parameters

    imu->setSlerpPower(0.02);
    imu->setGyroEnable(true);
    imu->setAccelEnable(true);
    imu->setCompassEnable(true);

    //  set up pressure sensor

    if (pressure != NULL)
        pressure->pressureInit();

    //  set up humidity sensor

    if (humidity != NULL)
        humidity->humidityInit();

    //  set up for rate timer

    rateTimer = displayTimer = RTMath::currentUSecsSinceEpoch();

  return RTC::RTC_OK;
}


RTC::ReturnCode_t Pi_RTIMU::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t Pi_RTIMU::onExecute(RTC::UniqueId ec_id)
{
        usleep(imu->IMUGetPollInterval() * 1000);

        while (imu->IMURead()) {
            RTIMU_DATA imuData = imu->getIMUData();

            //  add the pressure data to the structure

            if (pressure != NULL)
                pressure->pressureRead(imuData);

            //  add the humidity data to the structure

            if (humidity != NULL)
                humidity->humidityRead(imuData);

            sampleCount++;

            now = RTMath::currentUSecsSinceEpoch();

            //  display 5 times per second

            if ((now - displayTimer) > 200000) {
#ifdef _DEBUG
              printf("Sample rate %d: %s\n",
                     sampleRate,
                     RTMath::displayDegrees("", imuData.fusionPose));
#endif
              m_accel.data.ax = imuData.accel.x();
              m_accel.data.ay = imuData.accel.y();
              m_accel.data.az = imuData.accel.z();
              m_accelOut.write();
              m_compass.data.x = imuData.compass.x();
              m_compass.data.y = imuData.compass.y();
              m_compass.data.z = imuData.compass.z();
              m_compassOut.write();
              m_gyro.data.aax = imuData.gyro.x();
              m_gyro.data.aay = imuData.gyro.y();
              m_gyro.data.aaz = imuData.gyro.z();
              m_gyroOut.write();
              m_pose.data.r = imuData.fusionPose.x();
              m_pose.data.p = imuData.fusionPose.y();
              m_pose.data.y = imuData.fusionPose.z();
              m_poseOut.write();
              m_qpose.data.x = imuData.fusionQPose.x();
              m_qpose.data.y = imuData.fusionQPose.y();
              m_qpose.data.z = imuData.fusionQPose.z();
              m_qpose.data.w = imuData.fusionQPose.scalar();
              m_qposeOut.write();

              if (pressure != NULL)
              {
#ifdef _DEBUG
                  printf("Pressure: %4.1f, height above sea level: %4.1f, temperature: %4.1f",
                         imuData.pressure, RTMath::convertPressureToHeight(imuData.pressure), imuData.temperature);
#endif
                  m_temperature.data = imuData.temperature;
                  m_temperatureOut.write();
                  m_pressure.data = imuData.pressure;
                  m_pressureOut.write();

                }
                if (humidity != NULL) {
#ifdef _DEBUG
                  printf(", humidity: %4.1f",
                         imuData.humidity);
#endif
                  m_humidity.data = imuData.humidity;
                  m_humidityOut.write();
                }
#ifdef _DEBUG
                printf("\n");
                fflush(stdout);
#endif
                displayTimer = now;
            }

            //  update rate every second

            if ((now - rateTimer) > 1000000) {
                sampleRate = sampleCount;
                sampleCount = 0;
                rateTimer = now;
            }
        }
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t Pi_RTIMU::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Pi_RTIMU::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Pi_RTIMU::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Pi_RTIMU::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Pi_RTIMU::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void Pi_RTIMUInit(RTC::Manager* manager)
  {
    coil::Properties profile(pi_rtimu_spec);
    manager->registerFactory(profile,
                             RTC::Create<Pi_RTIMU>,
                             RTC::Delete<Pi_RTIMU>);
  }
  
};


